/* $USAGI: ipsec4_output.c,v 1.1 2002/07/26 08:08:32 miyazawa Exp $ */
/*
 * Copyright (C)2001 USAGI/WIDE Project
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <asm/byteorder.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/sysctl.h>
#include <linux/inet.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include <linux/smp.h>
#include <linux/list.h>
#include <asm/uaccess.h>
#include <net/ipv6.h>
#include <net/sadb.h>
#include <net/spd.h>
#include <net/addrconf.h>
#include <net/snmp.h>  
#include <linux/in.h>
#include <linux/ipsec.h>
#include <linux/ipsec6.h>
#include <net/pfkeyv2.h> /* sa proto type */
#include <net/pfkey.h>
#include <net/ipcomp.h>


static int ipsec4_output_check_core(struct selector *selector, struct ipsec_sp **policy_ptr)
{
	int error = 0;
	struct ipsec_sp *policy = NULL;
	int result = IPSEC_ACTION_BYPASS; 	/* default */

	IPSEC4_DEBUG("called\n");

	if (!selector) {
		IPSEC4_DEBUG("selector is NULL\n");
		error = -EINVAL;
		goto err;
	}

	policy = spd_get(selector);
	if (!policy) { /* not match ! */
		IPSEC4_DEBUG("no policy exists.\n");
		result = IPSEC_ACTION_BYPASS;
		goto err;
	}

	read_lock_bh(&policy->lock);
	if (policy->policy_action == IPSEC_POLICY_DROP) {
		result = IPSEC_ACTION_DROP;
		read_unlock_bh(&policy->lock);
		goto err;
	}  else if (policy->policy_action == IPSEC_POLICY_BYPASS) {
		result = IPSEC_ACTION_BYPASS;
		read_unlock_bh(&policy->lock);
		goto err;
	}
	
	/* policy must then be to apply ipsec */
	if (policy->auth_sa_idx) {
		if (policy->auth_sa_idx->sa) {
			struct ipsec_sa *sa = NULL;
			ipsec_sa_hold(policy->auth_sa_idx->sa);
			sa = policy->auth_sa_idx->sa;
			read_unlock_bh(&policy->lock);

			read_lock_bh(&sa->lock);
			switch (sa->state) {
			case SADB_SASTATE_MATURE:
			case SADB_SASTATE_DYING:
				result |= IPSEC_ACTION_AUTH;
				break;
			default:
				result = IPSEC_ACTION_DROP;
			}
			read_unlock_bh(&sa->lock);
			ipsec_sa_put(sa);
		} else {
			/* copy sa_idx in policy to avoid to lock SADB and SPD at the same time */
			struct sa_index sa_idx;
			sa_index_init(&sa_idx);
			sa_index_copy(&sa_idx, policy->auth_sa_idx);
			read_unlock_bh(&policy->lock);

			sa_idx.sa = sadb_find_by_sa_index(&sa_idx);
			if (sa_idx.sa) {
				write_lock_bh(&policy->lock);
				policy->auth_sa_idx->sa = sa_idx.sa;
				ipsec_sa_hold(policy->auth_sa_idx->sa);
				write_unlock_bh(&policy->lock);
				ipsec_sa_put(sa_idx.sa);
				result |= IPSEC_ACTION_AUTH;
			} else {
				/* SADB_ACQUIRE message should be thrown up to KMd */
				result = IPSEC_ACTION_DROP;
			}
		}
	} else {
		read_unlock_bh(&policy->lock);
	}

	read_lock_bh(&policy->lock);
	if (policy->esp_sa_idx) {
		if (policy->esp_sa_idx->sa) {
			struct ipsec_sa *sa = NULL;
			ipsec_sa_hold(policy->esp_sa_idx->sa);
			sa = policy->esp_sa_idx->sa;
			read_unlock_bh(&policy->lock);

			read_lock_bh(&sa->lock);
			switch (sa->state) {
			case SADB_SASTATE_MATURE:
			case SADB_SASTATE_DYING:
				result |= IPSEC_ACTION_ESP;
				break;
			default:
				result = IPSEC_ACTION_DROP;
			}
			read_unlock_bh(&sa->lock);
			ipsec_sa_put(sa);
		} else {
			/* copy sa_idx in policy to avoid to lock SADB and SPD at the same time */
			struct sa_index sa_idx;
			sa_index_init(&sa_idx);
			sa_index_copy(&sa_idx, policy->esp_sa_idx);
			read_unlock_bh(&policy->lock);

			sa_idx.sa = sadb_find_by_sa_index(&sa_idx);
			if (sa_idx.sa) {
				write_lock_bh(&policy->lock);
				policy->esp_sa_idx->sa = sa_idx.sa;
				ipsec_sa_hold(policy->esp_sa_idx->sa);
				write_unlock_bh(&policy->lock);
				ipsec_sa_put(sa_idx.sa);
				result |= IPSEC_ACTION_ESP;
			} else {
				/* SADB_ACQUIRE message should be thrown up to KMd */
				result = IPSEC_ACTION_DROP;
			}
		}
	} else {
		read_unlock_bh(&policy->lock);
	}

	read_lock_bh(&policy->lock);
	if (policy->comp_sa_idx) {
		if (policy->comp_sa_idx->sa) {
			struct ipsec_sa *sa = NULL;
			ipsec_sa_hold(policy->comp_sa_idx->sa);
			sa = policy->comp_sa_idx->sa;
			read_unlock_bh(&policy->lock);

			read_lock_bh(&sa->lock);
			switch (sa->state) {
			case SADB_SASTATE_MATURE:
			case SADB_SASTATE_DYING:
				result |= IPSEC_ACTION_COMP;
				break;
			default:
				result |= IPSEC_ACTION_DROP;
			}
			read_unlock_bh(&sa->lock);
			ipsec_sa_put(sa);
		} else {
			/* copy sa_idx in policy to avoid to lock SADB and SPD at the same time */
			struct sa_index sa_idx;
			sa_index_init(&sa_idx);
			sa_index_copy(&sa_idx, policy->comp_sa_idx);
			read_unlock_bh(&policy->lock);

			sa_idx.sa = sadb_find_by_sa_index(&sa_idx);
			if (sa_idx.sa) {
				write_lock_bh(&policy->lock);
				policy->comp_sa_idx->sa = sa_idx.sa;
				ipsec_sa_hold(policy->comp_sa_idx->sa);
				write_unlock_bh(&policy->lock);
				ipsec_sa_put(sa_idx.sa);
				result |= IPSEC_ACTION_COMP;
			} else {
				/* SADB_ACUIRE message should be thrown up to KMd */
				result |= IPSEC_ACTION_DROP;
			}
		}
	} else {
		read_unlock_bh(&policy->lock);
	}

	*policy_ptr= policy;

	IPSEC4_DEBUG("end\n");	

err:
	return result;
}

int ipsec4_output_check(struct sock *sk,  struct rtable *rt, struct ipsec_sp **policy_ptr)
{
	struct in_addr saddr,daddr;
	u16 sport,dport;
	unsigned char proto;
	struct selector selector;
	int result = IPSEC_ACTION_BYPASS; 	/* default */

	IPSEC4_DEBUG("called\n");
	if (!sk) {
		printk(KERN_ERR "flowi and sock are NULL\n");
		result = -EINVAL;
		goto err;
	}
	
	if (rt) {
		saddr.s_addr = rt->rt_src;
	}else if (sk) {
		saddr.s_addr = sk->saddr; 
	} else {
		result = -EINVAL;
		goto err;
	}

	if (rt) {
		daddr.s_addr = rt->rt_dst;
	} else if (sk) {
		daddr.s_addr = sk->daddr;
	} else { 
		result = -EINVAL;
		goto err;
	}

	if (sk) {
		sport=sk->sport;
		dport=sk->dport;
		proto=sk->protocol;
	} else {
		result = -EINVAL;
		goto err;
	}

	/* for ISKAMP see RFC2408 */
	if (proto == IPPROTO_UDP && 
	    sport == __constant_htons(500) && dport == __constant_htons(500)) {
		result = IPSEC_ACTION_BYPASS; 	/* default */
		goto err;
	}

	if (proto == IPPROTO_ICMP) {
		sport = 0;
		dport = 0;
	}

	memset(&selector, 0, sizeof(struct selector));

	((struct sockaddr_in *)&selector.src)->sin_family = AF_INET;
	((struct sockaddr_in *)&selector.src)->sin_addr = saddr;
	((struct sockaddr_in *)&selector.dst)->sin_family = AF_INET;
	((struct sockaddr_in *)&selector.dst)->sin_addr = daddr;
	selector.proto = proto;
	selector.prefixlen_d = 32;
	selector.prefixlen_s = 32;

	((struct sockaddr_in *)&selector.src)->sin_port = sport;	
	((struct sockaddr_in *)&selector.dst)->sin_port = dport;	

#ifdef CONFIG_IPSEC_DEBUG
	{
		char buf[64];
		IPSEC4_DEBUG("original src addr: %s\n", in_ntop(&saddr, buf));
		IPSEC4_DEBUG("original src port: %u\n", ntohs(sport));
		IPSEC4_DEBUG("original dst addr: %s\n", in_ntop(&daddr, buf));
		IPSEC4_DEBUG("original dst port: %u\n", ntohs(dport));

		IPSEC4_DEBUG("selector src addr: %s\n", 
				in_ntop( &((struct sockaddr_in *)&selector.src)->sin_addr, buf));
		IPSEC4_DEBUG("selector src port: %u\n", 
				ntohs(((struct sockaddr_in *)&selector.src)->sin_port));
		IPSEC4_DEBUG("selector dst addr: %s\n", 
				in_ntop( &((struct sockaddr_in *)&selector.dst)->sin_addr, buf));
		IPSEC4_DEBUG("selector dst port: %u\n", 
				ntohs(((struct sockaddr_in *)&selector.dst)->sin_port));
		IPSEC4_DEBUG("selector proto: %u\n", selector.proto);
	}
#endif /* CONFIG_IPSEC_DEBUG */

	result = ipsec4_output_check_core(&selector, policy_ptr);

 err:
		return result;
}

